@@ -5,12 +5,28 @@ use warnings;
 
 use parent qw{ HTML::Parser };
 
-our $VERSION = '0.201';
+our $VERSION = '0.300';
 
+our $CLASSIC = 0;
 our $DEBUG = 0;
 
 my @stacked = qw{ current_table current_row current_element };
 
+sub new
+{
+	my ( $class, %arg ) = @_;
+	my $classic = delete $arg{classic};
+	my $self = $class->SUPER::new( %arg );
+	$self->{ATTR}{classic} =  defined $classic ? $classic : $CLASSIC;
+	return $self;
+}
+
+sub classic
+{
+	my ( $self ) = @_;
+	return $self->{ATTR}{classic};
+}
+
 sub start
 {
 #	my ($self, $tag, $attr, $attrseq, $origtext) = @_;
@@ -31,6 +47,10 @@ sub start
 	} elsif ($tag eq 'th') {
 		my $th = $attr;
 		push @{$self->{STORE}->{current_table}->{headers}}, $th;
+		unless ( $self->{ATTR}{classic} ) {
+			push @{$self->{STORE}->{current_row}->{cells}}, undef;
+			push @{$self->{STORE}->{current_row}->{headers}}, $th;
+		}
 		$self->{STORE}->{current_element} = $th;
 
 	} elsif ($tag eq 'tr') {
@@ -42,6 +62,9 @@ sub start
 	} elsif ($tag eq 'td') {
 		my $td = $attr;
 		push @{$self->{STORE}->{current_row}->{cells}}, $td;
+		unless ( $self->{ATTR}{classic} ) {
+			push @{$self->{STORE}->{current_row}->{headers}}, undef;
+		}
 		$self->{STORE}->{current_element} = $td;
 
 	} elsif ($tag eq 'caption') {
@@ -94,10 +117,14 @@ sub end
 		$self->{STORE}{$_} = $prev->{$_} for @stacked;
 
 	} elsif ($tag eq 'th') {
-		$self->{STORE}->{current_row} = undef;
 		$self->{STORE}->{current_element} = undef;
-
 	} elsif ($tag eq 'tr') {
+		for my $key ( 'cells', $self->{ATTR}{classic} ? () : 'headers' ) {
+			my $data = $self->{STORE}{current_row}{$key} || [];
+			pop @{ $data } while @{ $data } && !  $data->[-1];
+			delete $self->{STORE}{current_row}{$key}
+				unless @{ $data };
+		}
 		$self->{STORE}->{current_row} = undef;
 		$self->{STORE}->{current_element} = undef;
 
@@ -169,15 +196,23 @@ HTML::TableContentParser - Do interesting things with the contents of tables.
 =head1 SYNOPSIS
 
   use HTML::TableContentParser;
-  $p = HTML::TableContentParser->new();
-  $tables = $p->parse($html);
+  my $p = HTML::TableContentParser->new();
+  my $html = read_html_from_somewhere();
+  my $tables = $p->parse_file( $html );
+  for my $t (@$tables) {
+    for my $r (@{$t->{rows}}) {
+      print 'Row:';
+      for my $c (@{$r->{cells}}) {
+        print " [$c->{data}]";
+      }
+      print "\n";
+    }
+  }
 
 =head1 DESCRIPTION
 
-This package pulls out the contents of a table from a string containing HTML.
-Each time a table is encountered, data will be stored in an array consisting
-of a hash of whatever was discovered about the table -- id, name, border,
-cell spacing etc, and of course data contained within the table.
+This package parses tables out of HTML. The return from the parse is a
+reference to an array containing the tables found.
 
 Tables appear in the output in the order in which they are encountered.
 If a table is nested inside a cell of another table, it will appear
@@ -185,86 +220,171 @@ after the containing table in the output, and any connection between the
 two will be lost. As of version 0.200_01, the appearance of a nested
 table should not cause any truncation of the containing table.
 
-The format of each hash will look something like
+The following tags are processed by this module: C<< <table> >>,
+C<< <caption> >>, C<< <tr> >>, C<< <th> >>, and C<< <td> >>. In the
+return from the parse method, each tag is represented by a hash
+reference, having the tag's attributes as keys, and the attribute values
+as values. In addition, the following keys will be provided:
 
-  attributes            keys from the attributes of the <table> tag
-  @{$table_headers}     array of table headers, in order found
-  @{$table_rows}        rows discovered, in order
+=over
 
-If the table has a caption, this will be provided as
+=item C<< <table> >>
 
-  caption               keys from the caption tag's attributes
-    data                the text of the <caption>..</caption> element
+=over
 
-then for each table row,
-    @{$table_data}      td's found, in order
-    other attributes    the ... in <tr ...>
+=item caption
 
-then for each data cell,
-      data              what comes between <td> and </td>
-      other attributes  the ... in <td ...>
+the C<< <caption> >> tag, if any
 
+=item headers
 
-=head2 EXAMPLE
+a reference to an array containing all the C<< <th> >> tags, in the
+order encountered
 
-  use HTML::TableContentParser;
-  $p = HTML::TableContentParser->new();
-	$html = read_html_from_somewhere();
-  $tables = $p->parse($html);
-  for $t (@$tables) {
-    for $r (@{$t->{rows}}) {
-			print "Row: ";
-      for $c (@{$r->{cells}}) {
-        print "[$c->{data}] ";
-      }
-      print "\n";
-    }
-  }
+=item rows
 
-
-=head1 METHODS
-
-=over 4
-
-=item start($parser, $tag, $attr, $attrseq, $origtext);
-
-Called whenever a particular start tag has been recognised. This is called
-automatically by the parser and should not be called from the application.
-
-
-=item text($parser, $content);
-
-Called whenever a piece of content is encountered. This is called
-automatically by the parser and should not be called from the application.
-
-=item end($parser, $tag, $origtext);
-
-Called whenever a particular end tag is encountered. This is called
-automatically by the parser and should not be called from the application.
-
-=item $tables_ref = $p->parse($html);
-
-Called with the HTML to parse. This is all the application needs to do.
-The return value will be an arrayref containing each table encountered, in the
-format detailed above.
-
-This method will C<croak()> if the argument is not defined, or not
-specified.
-
-=item DEBUG
-
-Not a method, but a class variable. Set to 1 to cause debugging output
-(basically the structure and content of the table) to be sent to stdout via
-warn().
+a reference to an array containing all the C<< <tr> >> tags, in the
+order encountered
 
 =back
 
-=head2 EXPORTS
+=item C<< <caption> >>
+
+=over
+
+=item data
+
+the content of the C<< <caption> >> tag
+
+=back
+
+=item C<< <tr> >>
+
+=over
+
+=item cells
+
+a reference to an array containing all the C<< <td> >> tags, in the
+order encountered, with C<undef> representing any C<< <th> >> tags
+encountered. Trailing C<undef> values will be dropped, and the entire
+key will be absent unless actual C<< <td> >> tags are found in the row.
+
+Note that prior to version 0.299_01, C<< <th> >> tags were not
+represented at all.
+
+=item headers
+
+new with version 0.299_01, this is a reference to an array containing all the
+C<< <th> >> tags in the row, in the order encountered, with C<undef>
+representing any C<< <td> >> tags. Trailing C<undef> values will be
+dropped, and the entire key will be absent unless actual C<< <th> >>
+tags are found in the row.
+
+It is the understanding of the current author (TRW) that in valid HTML
+C<< <th> >> tags must occur inside a C<< <tr> >> element, so they need
+to be recognized there, rather than (or in addition to) in isolation.
+
+=back
+
+=item C<< <th> >>
+
+=over
+
+=item data
+
+the content of the C<< <th> >> tag
+
+=back
+
+=item C<< <td> >>
+
+=over
+
+=item data
+
+the content of the C<< <td> >> tag
+
+=back
+
+=back
+
+=head1 METHODS
+
+This module is a subclass of L<HTML::Parser|HTML::Parser>. It provides
+only one new method, L<classic()|/classic>, which is an accessor for the
+attribute of the same name.  The following inherited (or overridden)
+methods may profitably be called by the user.
+
+=head2 new
+
+ my $p = HTML::TableContentParser->new();
+
+This static method instantiates the parser object. The only supported
+argument is
+
+=over
+
+=item classic
+
+If this argument is set to C<1>, C<< <th> >> tags are handled in the
+pre-0.299_01 way. That is, the C<< <tr> >> hash will not
+contain a C<{headers}> key, and its C<{cells}> key will not contain any
+C<undef> values corresponding to C<< <th> >> elements.
+
+If this argument is set to C<0>, you get the behavior documented for
+0.299_01 and after.
+
+If this argument is C<undef> or omitted, the value of
+L<$HTML::TableContentParser::CLASSIC|/HTML::TableContentParser::CLASSIC>
+is used.
+
+No other values are supported -- that is, the author reserves them, and
+the behavior when you use them may change without warning.
+
+=back
+
+=head2 classic
+
+This method returns the value of the C<classic> attribute, whether
+specified or defaulted.
+
+=head2 parse
+
+ my $tables = $p->parse( $html );
+
+This method parses the given HTML. The return is a reference to an array
+containing all the tables found.
+
+=head1 GLOBALS
+
+The following global variables, B<properly localized,> can be used to
+modify the behavior of this module.
+
+=head2 $HTML::TableContentParser::CLASSIC
+
+This variable provides the default value of the C<classic> argument to
+L<new()|/new>, and is subject to the same restrictions.
+
+=head2 $HTML::TableContentParser::DEBUG
+
+If set to C<1>, causes debug output to F<STDERR> (via C<warn()>).
+Setting this to any true value (including C<1>) is unsupported in the
+sense that the behavior of this module in response to any true value is
+explicitly undocumented, and can change without notice.
+
+=head1 EXPORTS
 
 Nothing.
 
+=head1 CAVEATS, BUGS, and TODO
 
-=head2 CAVEATS, BUGS, and TODO
+The C<rowspan> and C<colspan> attributes are reported but ignored. That
+is,
+
+ <tr><td colspan="2">Moe</td><td>Howard</td></tr>
+
+occupies three columns in the HTML table, but only two entries are made
+in the C<{cells}> value of the hash that represents this row.
 
 =head1 SEE ALSO
 
